#include <vector>
#include <algorithm>
#include <set>

#include<vcg/simplex/vertexplus/base.h>
#include<vcg/simplex/faceplus/base.h>
#include<vcg/complex/trimesh/base.h>

#include<vcg/complex/trimesh/clean.h>
#include<vcg/complex/trimesh/update/topology.h>

#include<wrap/io_trimesh/import_ply.h>

using namespace vcg;

class MyEdge;    // dummy prototype never used
class MyFace;
class MyVertex;

class MyVertex  : public VertexSimp2< MyVertex, MyEdge, MyFace, vertex::Coord3f, vertex::BitFlags  >{};
class MyFace    : public FaceSimp2  < MyVertex, MyEdge, MyFace, face::VertexRef, face::FFAdj, face::BitFlags > {};

class MyMesh : public tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace > >{};


int main(int argc,char **argv ){

	MyMesh m;

	if(tri::io::ImporterPLY<MyMesh>::Open(m,argv[1])!=0)
		{
      printf("Error reading file  %s\n",argv[1]);
			exit(0);
		}
		
	printf("vn %i fn %i\n",m.vn, m.fn);

	tri::UpdateTopology<MyMesh>::FaceFace(m);
	
	if(tri::Clean<MyMesh>::IsTwoManifoldFace(m) ) 
		printf("Mesh is two manifold\n");
	else
		printf("Mesh is NOT two manifold\n");
	
	int e0bit = MyFace::NewBitFlag();
	int e1bit = MyFace::NewBitFlag();
	int e2bit = MyFace::NewBitFlag();
	int ebit[3] = {e0bit,e1bit,e2bit};
	
	MyMesh::FaceIterator fi;
	int BorderCount =0;
	int InternalCount =0;
	int NonManifCount =0;
	for(fi=m.face.begin();fi!=m.face.end();++fi)
	{
		for(int i=0;i<3;++i)
			(*fi).ClearUserBit(ebit[i]);
	}		
	
	for(fi=m.face.begin();fi!=m.face.end();++fi)
	{
		for(int i=0;i<3;++i)
		{
			MyMesh::FacePointer fa = (*fi).FFp(i);
			int ifa =  (*fi).FFi(i);
			
			if( fa == &*fi ) BorderCount++;
			else
			{
				if( fa->FFp(ifa) == &*fi ) 
				{
					InternalCount++;
					assert(fa->FFi(ifa) == i);
				}
				else
				{
					if(!(*fi).IsUserBit(ebit[i]) )
						{
							NonManifCount++;
						  (*fi).SetUserBit(ebit[i]);
							while(fa != &*fi )
							{
								fa->SetUserBit(ebit[ifa]);
								MyMesh::FacePointer newfa = fa->FFp(ifa);
								ifa = fa->FFi(ifa);
								fa=newfa;
							}
							assert(i==ifa);
						}
				}
			}
		}
	}
	
	printf("Found %i Borders, %i internal, %i non manif\n",BorderCount,InternalCount/2,NonManifCount);

  return 0;
}

